/*
----------------------
Be Sample Code License
----------------------

Copyright 1991-2001, Be Incorporated.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions, and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions, and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the author may not be used to endorse or promote products
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR "AS IS" AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF TITLE, NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.    
*/
#include "debug.h"

#ifndef _netcentral
#define _netcentral

#include <List.h>
#include <Looper.h>
#include <MessageRunner.h>
#include <NetEndpoint.h>
#include <OS.h>
#include <Window.h>

#define DASPORTAL_DEFAULT_BCAST_PORT 0x2710
#define DASPORTAL_FILE_LIST 'list'
#define DASPORTAL_GET_FILE 'gfle'
#define DASPORTAL_FILE_REQUEST 'freq'
#define DASPORTAL_ADD_HOST 'ahst'
#define DASPORTAL_REMOVE_HOST 'rhst'
#define DASPORTAL_SEND_TO 'snd2'

class NetCentral: public BLooper {
	private:
	//Private Data Type
		struct thmaster {
			thread_id thread;
			BMessage *msg;
		};
		
	//Private Variables
		BList *threadlist, *queue;
		BMessageRunner *mqchecker;
		BNetEndpoint *mBroadcast_ep;
		thread_id mReceive_th, mFRC_th;
		sem_id mConstructor_sem, mQueueList_sem,
				mThreadList_sem, mFS_sem, mFR_sem;
		volatile uint8 quit;
		volatile int32 fscount, frcount;
	public:
	//Public Variables
		BWindow *MainWindow;
		
	//Public Functions
		//BLooper Inherited Functions
			NetCentral(const char *name);
			~NetCentral();
			bool QuitRequested();
			void MessageReceived(BMessage *msg);
		
		//Class Specific Functions
			int32 Broadcast(BMessage *msg, uint16 port = DASPORTAL_DEFAULT_BCAST_PORT);
			int32 SendTo(BMessage *msg,const char *host,uint16 port = DASPORTAL_DEFAULT_BCAST_PORT);

		//Thread Related Functions
			static int32 SpawnReceiverThread(void *arg);
			static int32 SpawnFileSender(thmaster *mystuff);
			static int32 SpawnFileReceiverThread(BNetEndpoint *ep);
			static int32 SpawnFRConnection(void *arg);
			int32 ReceiverThread();
			int32 FileSenderThread(thmaster *mystuff);
			int32 FileReceiverThread(BNetEndpoint *ep);
			int32 FRConnection();
};

#endif
