/*
----------------------
Be Sample Code License
----------------------

Copyright 1991-2001, Be Incorporated.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions, and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions, and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the author may not be used to endorse or promote products
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR "AS IS" AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF TITLE, NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.    
*/
#include <Entry.h>
#include <Path.h>
#include <ScrollView.h>
#include <string.h>
//#include <translation/TranslationUtils.h>
#include "netcentral.h"
#include "topview.h"
#include "win.h"

extern BMessage *SettingsMsg;

#define Remove_Files 'rmfl'
#define Download_Files 'dlfl'

flstfilter::flstfilter(BView *vw): BMessageFilter(B_ANY_DELIVERY, B_ANY_SOURCE) {
	view = vw;
}


filter_result flstfilter::Filter(BMessage *msg, BHandler **target) {
	if (msg->what == DASPORTAL_FILE_LIST)
		PRINT(("message filter\n"));
	if (msg->what == DASPORTAL_FILE_LIST) {
	if (acquire_sem(((topview*)view)->update_sem)!=B_OK)
		return B_SKIP_MESSAGE;
	PRINT(("msgfilter\n"));
	msg->PrintToStream();
		Win *win = (Win*) (view->Window());
		siteinfo_st *tmp;
		BString host;
		msg->FindString("host", &host);
		for (int32 i = 0; i < win->HostList->CountItems(); i++) {
			tmp = (siteinfo_st*) win->HostList->ItemAt(i);
			if (tmp->host == host) {
				PRINT(("deleting old file list\n"));
				tmp->files->PrintToStream();
				if (tmp->files!=NULL)
					delete tmp->files;
				tmp->files = NULL;
				tmp->files = new BMessage(*msg);
				tmp->files->PrintToStream();
				if (!tmp->files->HasString("host"))
					tmp->files->AddString("host", tmp->host.String());
				break;
			}
		}
	*target = view;
	release_sem(((topview*)view)->update_sem);
	}
	return B_DISPATCH_MESSAGE;
}


topview::topview(BRect frame, const char *name, uint32 resizem, uint32 flags):
	BView(frame,name,  resizem, flags) {
	NetCenter = NULL;
	update_sem=create_sem(1,"update_sem");
	BRect r(Bounds());
	r.InsetBy(10, 10);
	box = new BBox(r, "box", B_FOLLOW_ALL, B_WILL_DRAW | B_FRAME_EVENTS | B_NAVIGABLE_JUMP, B_FANCY_BORDER);
	BMessage *msg = new BMessage(MySharedFiles);
	BMenuItem *mitem = new BMenuItem("My Shared Files", msg);
	menu = new BMenu("menubox", B_ITEMS_IN_COLUMN);
	menu->SetLabelFromMarked(true);
	menu->SetRadioMode(true);
	menu->AddItem(mitem);
	mitem->SetMarked(true);
	menu->AddSeparatorItem();
	mfield = new BMenuField(Bounds(), "menufield", "Select:", menu, B_FOLLOW_LEFT, B_WILL_DRAW | B_NAVIGABLE);
	mfield->SetDivider(40);
	box->SetLabel(mfield);
	cur_host = NULL;
	AddChild(box);
	r = box->Bounds();
	r.top += 25;
	r.left += 5;
	r.bottom -= 5;
	r.right /= 2;
	lview = new BListView(r, "files", B_MULTIPLE_SELECTION_LIST, B_FOLLOW_ALL, B_WILL_DRAW | B_NAVIGABLE | B_FRAME_EVENTS);
	box->AddChild((new BScrollView("scroller", lview, B_FOLLOW_ALL, B_WILL_DRAW | B_NAVIGABLE_JUMP, false, true)));
	r = box->Bounds();
	r.bottom /= 2.0;
	r.top = r.bottom - 20;
	r.right -= 20;
	r.left = r.right - 80;
	msg = new BMessage(Remove_Files);
	remove = new BButton(r, "removefiles", "Remove", msg, B_FOLLOW_RIGHT, B_WILL_DRAW | B_NAVIGABLE);
	box->AddChild(remove);
	r.top += 25;
	r.bottom += 25;
	msg = new BMessage(Download_Files);
	dload=new BButton(r, "downloadbutton", "Download", msg, B_FOLLOW_RIGHT,B_WILL_DRAW | B_NAVIGABLE);
	box->AddChild(dload);
	dload->SetEnabled(false);
	AddFilter((new flstfilter(this)));
}


topview::~topview() {
	delete_sem(update_sem);
}

void topview::MessageReceived(BMessage *msg) {
	#if DEBUG
	PRINT(("topview::MessageReceived\nMessage Received:\n"));
	msg->PrintToStream();
	#endif
	switch(msg->what) {
		case Remove_Files: {
			if (lview->CurrentSelection() >= 0) {
				BMessage *remover = new BMessage(B_SIMPLE_DATA);
				remover->AddInt32("buttons", B_SECONDARY_MOUSE_BUTTON);
				int32 selected = 0,current = 0;
				BEntry ent;
				entry_ref ref;
				while ((selected = lview->CurrentSelection(current)) >= 0) {
					ent.SetTo((((BStringItem*)lview->ItemAt(selected))->Text()), false);
					ent.GetRef(&ref);
					remover->AddRef("refs", &ref);
					current++;
				}
				BMessenger *msgr = new BMessenger(NULL, Window(), NULL);
				msgr->SendMessage(remover);
				delete msgr;
				delete remover;
			}
			}
			break;
		case Download_Files: {
			if (lview->CurrentSelection() >= 0) {
				BMessage *dlmsg = NULL, *sndto = NULL;
				int32 selected = 0,current = 0;
				entry_ref ref;
				BString fname;
				BMessenger *msgr = new BMessenger(NULL, NetCenter, NULL);
				while((selected = lview->CurrentSelection(current)) >= 0) {
					dlmsg = new BMessage(DASPORTAL_FILE_REQUEST);
					sndto = new BMessage(DASPORTAL_SEND_TO);
					sndto->AddString("target", cur_host->host.String());
					sndto->AddInt16("port",DASPORTAL_DEFAULT_BCAST_PORT);
					cur_host->files->FindRef("refs", selected, &ref);
					dlmsg->AddRef("refs", &ref);
					cur_host->files->FindString("filename", selected, &fname);
					dlmsg->AddString("filename", fname.String());
					sndto->AddMessage("message", dlmsg);
					msgr->SendMessage(sndto);
					delete dlmsg;
					delete sndto;
					current++;
				}
				delete msgr;
			}
			}
			break;
		case DASPORTAL_FILE_LIST: {
			if (acquire_sem(update_sem)!=B_OK)
				return;
			if (cur_host == NULL) {
				if (!msg->HasString("host")) {
					entry_ref ref;
					BPath path;
					int32 count = 0;
					type_code code=B_ANY_TYPE;
					BMessage filelist;
					SettingsMsg->FindMessage("shared_files", &filelist);
					filelist.GetInfo("refs", &code, &count);
					LockLooper();
					while (!lview->IsEmpty())
						delete (BStringItem*) lview->RemoveItem(0L);
					for (int32 i = 0; i < count; i++) {
						filelist.FindRef("refs", i, &ref);
						path.SetTo(&ref);
						lview->AddItem((new BStringItem(path.Path())));
					}
					UnlockLooper();
				}
			} else {
				if (msg->HasString("host")) {
					BString host;
					msg->FindString("host", &host);
					if (host == cur_host->host) {
						int32 count = 0;
						type_code code;
						cur_host->files->GetInfo("refs", &code, &count);
						PRINT(("count: %ld\n",count));
						BString fname;
						LockLooper();
						while (!lview->IsEmpty())
							delete (BStringItem*) lview->RemoveItem(0L);
						msg->PrintToStream();
						cur_host->files->PrintToStream();
						PRINT(("list *should* be empty: items: %ld\n", lview->CountItems()));
						for (int32 i = 0; i < count; i++) {
							msg->FindString("filename", i, &fname);
							#if DEBUG
							PRINT(("\tAdding file %s to listview\n", fname.String()));
							#endif
							lview->AddItem((new BStringItem(fname.String())));
						}
						UnlockLooper();
						lview->Invalidate();
					}
				}
			}
			release_sem(update_sem);
			}
			break;
		case NetSharedFiles: {
			msg->FindPointer("siteinfo_st_ptr", (void**)&cur_host);
			if (cur_host != NULL) {
				#if DEBUG
				PRINT(("Displaying %s's shared files.\n", cur_host->host.String()));
				#endif
				int32 count = 0;
				type_code code = B_ANY_TYPE;
				cur_host->files->GetInfo("refs", &code, &count);
				BString fname;
				LockLooper();
				while (!lview->IsEmpty())
					delete (BStringItem*) lview->RemoveItem(0L);
				for (int32 i = 0; i < count; i++) {
					cur_host->files->FindString("filename", i, &fname);
					lview->AddItem((new BStringItem(fname.String())));
				}
				UnlockLooper();
				dload->SetEnabled(true);
				remove->SetEnabled(false);
			}
			}
			break;
		case MySharedFiles: {
			#if DEBUG
			PRINT(("Displaying local shared files.\n"));
			#endif
			cur_host = NULL;
			entry_ref ref;
			BPath path;
			int32 count=0;
			type_code code = B_ANY_TYPE;
			BMessage filelist;
			SettingsMsg->FindMessage("shared_files", &filelist);
			filelist.GetInfo("refs", &code, &count);
			LockLooper();
			while (!lview->IsEmpty())
				delete (BStringItem*) lview->RemoveItem(0L);
			for (int32 i = 0; i < count; i++) {
				filelist.FindRef("refs", i, &ref);
				path.SetTo(&ref);
				lview->AddItem((new BStringItem(path.Path())));
			}
			UnlockLooper();
			dload->SetEnabled(false);
			remove->SetEnabled(true);
			}
			break;
		default:
			BView::MessageReceived(msg);
	}
}


void topview::AllAttached() {
//	#if DEBUG
//	PRINT(("Adding nifty looking image\n"));
//	#endif
//	SetViewBitmap(BTranslationUtils::GetBitmap("bg_image"));
	SetViewColor(255, 255, 255);
	BMessenger *msgr = new BMessenger(this, NULL, NULL);
	msgr->SendMessage(MySharedFiles);
	delete msgr;
	remove->SetTarget(this);
	dload->SetTarget(this);
}
