/*
----------------------
Be Sample Code License
----------------------

Copyright 1991-2001, Be Incorporated.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions, and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions, and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the author may not be used to endorse or promote products
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR "AS IS" AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF TITLE, NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.    
*/
#include <Application.h>
#include <Entry.h>
#include <Messenger.h>
#include <Path.h>
#include <String.h>
#include "app.h"
#include "netcentral.h"
#include "win.h"

extern BMessage *SettingsMsg;

Win::Win(BRect frame, const char *title, window_type type, uint32 flags, uint32 wspaces): BWindow(frame, title, type, flags, wspaces) {
	#if DEBUG
	PRINT(("Win constructor\n"));
	#endif
	HostList = new BList(1);
	NetCenter = NULL;
	view = new topview(Bounds(), "background view", B_FOLLOW_ALL_SIDES, B_WILL_DRAW | B_ASYNCHRONOUS_CONTROLS);
//	AddFilter((new flstfilter(view)));
	AddChild(view);
}


Win::~Win() {
	#if DEBUG
	PRINT(("Win destructor\n"));
	#endif
	//play nice and deallocate everything properly...
	while (!HostList->IsEmpty())
		delete (siteinfo_st *) HostList->RemoveItem(0L);
	delete HostList;
}


bool Win::QuitRequested() {
	#if DEBUG
	PRINT(("Win QuitRequested\n"));
	#endif
	SettingsMsg->ReplaceRect("MainWindowFrame", Frame());
	return true;
}


void Win::MessageReceived(BMessage *msg) {
	#if DEBUG
	PRINT(("Win MessageReceived\n"));
	msg->PrintToStream();
	#endif
	switch(msg->what) {
		case B_SIMPLE_DATA: {
		/*
			We have functionality! This section of code will take dropped files and either allow you to send
			them to the file transfer window or add/remove them from the shared file list depending on if they
			were drag and dropped with the primary or secondary mouse button respectively.
		*/
			if (msg->HasRef("refs")) {
				int32 buttons = 0;
				msg->FindInt32("buttons", &buttons);
				switch(buttons) {
					case B_PRIMARY_MOUSE_BUTTON: { //handle dropped files
						if (view->cur_host != NULL) {
							int32 count = 0;
							type_code tcode = B_ANY_TYPE;
							entry_ref ref;
							BEntry ent;
							char fname[B_FILE_NAME_LENGTH];
							msg->GetInfo("refs", &tcode, &count);
							BMessage *filereq = new BMessage(DASPORTAL_GET_FILE), *requester = new BMessage(DASPORTAL_SEND_TO);
							BMessenger *msgr = new BMessenger(NULL, NetCenter, NULL);
							for (int32 i = 0; i < count; i++) {
								msg->FindRef("refs", i, &ref);
								ent.SetTo(&ref, false);
								memset(fname, 0, B_FILE_NAME_LENGTH);
								ent.GetName(fname);
								filereq->AddRef("refs", &ref);
								filereq->AddString("filename", fname);
								requester->AddString("target", view->cur_host->host.String());
								requester->AddInt16("port",DASPORTAL_DEFAULT_BCAST_PORT);
								requester->AddMessage("message", filereq);
								msgr->SendMessage(requester);
								delete filereq;
								delete requester;
								delete msgr;
							}
						}
						}
						break;
					case B_SECONDARY_MOUSE_BUTTON: {//add/remove dropped file(s); not recursive! that's a reader exercise
							BMessage filelist;
							bool nulist = false;
							if (SettingsMsg->HasMessage("shared_files")) {
								SettingsMsg->FindMessage("shared_files", &filelist);
							} else {
								filelist.what = DASPORTAL_FILE_LIST;
								nulist = true;
							}
							type_code tcode = B_ANY_TYPE;
							int32 countstored = 0, countnew = 0;
							entry_ref oldref, nuref;
							filelist.GetInfo("refs", &tcode, &countstored);
							if (countstored > 0) {//if we have stored refs, we need to remove any duplicate files dropped
								bool found = false;
								BEntry ent;
								BPath path;
								msg->GetInfo("refs", &tcode, &countnew);
								for (int32 i = 0; i < countnew; i++) {
									for (int32 j = 0; j < countstored; j++) {
										msg->FindRef("refs", i, &nuref);
										filelist.FindRef("refs", j, &oldref);
										if (nuref == oldref) {//file in the list, get rid of it
											#if DEBUG
											BString name;
											filelist.FindString("filename", j, &name);//Do you expect me to talk?
											PRINT(("Removing file \"%s\" from shared files list.\n", name.String()));
											#endif
											filelist.RemoveData("refs", j);//No, Mr. Bond, I expect you to die.
											filelist.RemoveData("filename", j);
											filelist.GetInfo("refs", &tcode, &countstored);
											found = true;
											break;
										}
									}
									if (!found) {
										ent.SetTo(&nuref, true);
										if (ent.Exists()) {
											ent.GetPath(&path);
											#if DEBUG //We are the Borg. 
											PRINT(("Adding file \"%s\" to shared files list.\n", path.Leaf()));
											#endif
											filelist.AddString("filename", path.Leaf());//You Will Be Assimilated.
											filelist.AddRef("refs", &nuref);//Resistance is futile.
										}
									}
								}
							} else { //No files stored, so just add these bad boys...
								entry_ref ref;
								type_code tcode;
								BEntry ent;
								BPath path;
								int32 count = 0;
								msg->GetInfo("refs", &tcode, &count);
								for (int32 i = 0;i < count; i++) {
									msg->FindRef("refs", i, &ref);
									ent.SetTo(&ref, true);
									if (ent.Exists()) {
										ent.GetPath(&path);
										#if DEBUG //We are the Borg.
										PRINT(("Adding file \"%s\" to shared files list.\n", path.Leaf()));
										#endif
										filelist.AddRef("refs", &ref); //You will service us.
										filelist.AddString("filename", path.Leaf()); //Resistance is futile.
									}
								}
							}
						if (nulist) {//make everything tidy again.
							SettingsMsg->AddMessage("shared_files", &filelist);
						} else {
							SettingsMsg->ReplaceMessage("shared_files", &filelist);
						}
						//Now spread the word about the update...
						BMessage *update = new BMessage(DASPORTAL_SEND_TO);
						siteinfo_st *tmp = NULL;
						for (int32 i = 0; i < HostList->CountItems(); i++){
							tmp = (siteinfo_st*) HostList->ItemAt(i);
							update->AddString("target", tmp->host.String());
							update->AddInt16("port",DASPORTAL_DEFAULT_BCAST_PORT);
						}
						update->AddMessage("message", &filelist);
						BMessenger *msgr = new BMessenger(NULL, NetCenter, NULL);
						msgr->SendMessage(update);
						delete msgr;
						msgr=new BMessenger(view, NULL, NULL);
						filelist.AddBool("local", true);
						msgr->SendMessage(&filelist);
						delete msgr;
						delete update;
						}
						break;
					}
			}
			}
			break;
		case DASPORTAL_ADD_HOST: {
				siteinfo_st *nu = new siteinfo_st, *tmp = NULL;
				msg->FindString("host", &nu->host);
				bool found = false;
				if (!HostList->IsEmpty()) {
					for (int32 i = 0; i < HostList->CountItems(); i++) {
						tmp = (siteinfo_st*) HostList->ItemAt(i);
						if (tmp->host == nu->host) {
							found = true;
							break;
						}
					}
				}
				if (found) {
					#if DEBUG
					PRINT(("Host %s is already in the host list.\n", tmp->host.String()));
					#endif
					delete nu;
				} else {
					#if DEBUG
					PRINT(("Host %s has been added to the host list.\n", nu->host.String()));
					#endif
					BMessage tmpmsg;
					msg->FindMessage("shared_files", &tmpmsg);//nu->files);
					nu->files = new BMessage(tmpmsg);
					nu->files->AddString("host", nu->host.String());
					nu->files->AddInt16("port",DASPORTAL_DEFAULT_BCAST_PORT);
					nu->files->PrintToStream();
					HostList->AddItem(nu);
				}
			}
			break;
		case DASPORTAL_REMOVE_HOST: {
				siteinfo_st *goner = new siteinfo_st, *tmp = NULL;
				msg->FindString("host", &goner->host);
				if (strcasecmp(goner->host.String(), (view->menu->FindMarked())->Label()) == 0) {
					view->menu->ItemAt(0)->SetMarked(true);
					BMessage *setthatpuppy = new BMessage(MySharedFiles);
					BMessenger *msgr = new BMessenger(view, NULL, NULL);
					msgr->SendMessage(setthatpuppy);
					delete msgr;
					delete setthatpuppy;
				}
				bool found = false;
				if (!HostList->IsEmpty()) {
					for (int32 i = 0; i < HostList->CountItems(); i++) {
						tmp = (siteinfo_st*) HostList->ItemAt(i);
						if (tmp->host == goner->host) {
							HostList->RemoveItem(i);
							found = true;
							break;
						}
					}
				}
				if (found) {
					#if DEBUG
					PRINT(("Host %s has been removed from the host list.\n", tmp->host.String()));
					#endif
					delete goner;
					delete tmp;
				} else {
					#if DEBUG
					PRINT(("Host %s is not in the host list. WTF?!?!\n", goner->host.String()));
					#endif
					delete goner;
				}
			}
			break;
		default:
			BWindow::MessageReceived(msg);
	}
}


void Win::MenusBeginning() {
	view->NetCenter = NetCenter;
	if (!HostList->IsEmpty()) {
		BMenuItem *item = NULL,*marked = view->menu->FindMarked();
		siteinfo_st *tmp = NULL;
		BMessage *msg = NULL;
		BString text;
		for (int32 i = 0; i < HostList->CountItems(); i++) {
			tmp = (siteinfo_st*) HostList->ItemAt(i);
			if (strcasecmp(tmp->host.String(), marked->Label()) == 0)
				continue;
			msg = new BMessage(NetSharedFiles);
			msg->AddPointer("siteinfo_st_ptr", tmp);
			item = new BMenuItem(tmp->host.String(), msg);
			view->menu->AddItem(item);
		}
		view->menu->SetTargetForItems(view);
	}
}


void Win::MenusEnded() {
	if (view->menu->CountItems() > 2) {
		BMenuItem *item = NULL, *marked = view->menu->FindMarked();
		int32 posit = 2L;
		while (view->menu->CountItems() > posit) {
			if (view->menu->ItemAt(posit) == marked) {
				posit++;
				continue;
			}
			item = view->menu->RemoveItem(posit);
			delete item;
		}
	}
}
